package ex.hm;

import bayonet.math.SpecialFunctions;
import blang.core.BlangExtensions;
import blang.core.ConstantSupplier;
import blang.core.DeboxedName;
import blang.core.IntVar;
import blang.core.Model;
import blang.core.ModelBuilder;
import blang.core.ModelComponent;
import blang.core.Param;
import blang.core.RealVar;
import blang.inits.Arg;
import blang.inits.DesignatedConstructor;
import blang.io.GlobalDataSource;
import blang.types.Index;
import blang.types.Plate;
import blang.types.Plated;
import blang.types.StaticUtils;
import blang.types.internals.RealScalar;
import ca.ubc.stat.blang.StaticJavaUtils;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.Optional;
import java.util.function.Supplier;

@SuppressWarnings("all")
public class GLM implements Model {
  public static class Builder implements ModelBuilder {
    private boolean fromCommandLine = false;
    
    @Arg
    public GlobalDataSource data;
    
    private boolean data_initialized = false;
    
    public GLM.Builder setData(final GlobalDataSource data) {
      data_initialized = true;
      this.data = data;
      return this;
    }
    
    @Arg
    public Plate<String> rocketType;
    
    private boolean rocketType_initialized = false;
    
    public GLM.Builder setRocketType(final Plate<String> rocketType) {
      rocketType_initialized = true;
      this.rocketType = rocketType;
      return this;
    }
    
    @Arg
    public Plate<Integer> launch;
    
    private boolean launch_initialized = false;
    
    public GLM.Builder setLaunch(final Plate<Integer> launch) {
      launch_initialized = true;
      this.launch = launch;
      return this;
    }
    
    @Arg
    public Plated<IntVar> success;
    
    private boolean success_initialized = false;
    
    public GLM.Builder setSuccess(final Plated<IntVar> success) {
      success_initialized = true;
      this.success = success;
      return this;
    }
    
    @Arg
    public Plated<RealVar> intercept;
    
    private boolean intercept_initialized = false;
    
    public GLM.Builder setIntercept(final Plated<RealVar> intercept) {
      intercept_initialized = true;
      this.intercept = intercept;
      return this;
    }
    
    @Arg
    public Plated<RealVar> slope;
    
    private boolean slope_initialized = false;
    
    public GLM.Builder setSlope(final Plated<RealVar> slope) {
      slope_initialized = true;
      this.slope = slope;
      return this;
    }
    
    @Arg
    public Optional<RealVar> interceptMean;
    
    public GLM.Builder setInterceptMean(final RealVar interceptMean) {
      // work around typeRef(..) limitation
      Optional<RealVar> $generated__dummy = null;
      this.interceptMean = Optional.of(interceptMean);
      return this;
    }
    
    @Arg
    public Optional<RealVar> interceptSD;
    
    public GLM.Builder setInterceptSD(final RealVar interceptSD) {
      // work around typeRef(..) limitation
      Optional<RealVar> $generated__dummy = null;
      this.interceptSD = Optional.of(interceptSD);
      return this;
    }
    
    @Arg
    public Optional<RealVar> slopeMean;
    
    public GLM.Builder setSlopeMean(final RealVar slopeMean) {
      // work around typeRef(..) limitation
      Optional<RealVar> $generated__dummy = null;
      this.slopeMean = Optional.of(slopeMean);
      return this;
    }
    
    @Arg
    public Optional<RealVar> slopeSD;
    
    public GLM.Builder setSlopeSD(final RealVar slopeSD) {
      // work around typeRef(..) limitation
      Optional<RealVar> $generated__dummy = null;
      this.slopeSD = Optional.of(slopeSD);
      return this;
    }
    
    @Arg
    public Optional<Map<String, RealVar>> prediction;
    
    public GLM.Builder setPrediction(final Map<String, RealVar> prediction) {
      // work around typeRef(..) limitation
      Optional<Map<String, RealVar>> $generated__dummy = null;
      this.prediction = Optional.of(prediction);
      return this;
    }
    
    public GLM build() {
      // For each optional type, either get the value, or evaluate the ?: expression
      if (!fromCommandLine && !data_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing data");
      final GlobalDataSource __data = data;
      if (!fromCommandLine && !rocketType_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing rocketType");
      final Plate<String> __rocketType = rocketType;
      if (!fromCommandLine && !launch_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing launch");
      final Plate<Integer> __launch = launch;
      if (!fromCommandLine && !success_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing success");
      final Plated<IntVar> __success = success;
      if (!fromCommandLine && !intercept_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing intercept");
      final Plated<RealVar> __intercept = intercept;
      if (!fromCommandLine && !slope_initialized)
        throw new RuntimeException("Not all fields were set in the builder, e.g. missing slope");
      final Plated<RealVar> __slope = slope;
      RealVar interceptMean;
      if (this.interceptMean != null && this.interceptMean.isPresent()) {
        interceptMean = this.interceptMean.get();
      } else {
        interceptMean = $generated__27(data, rocketType, launch, success, intercept, slope);
      }
      final RealVar __interceptMean = interceptMean;
      RealVar interceptSD;
      if (this.interceptSD != null && this.interceptSD.isPresent()) {
        interceptSD = this.interceptSD.get();
      } else {
        interceptSD = $generated__28(data, rocketType, launch, success, intercept, slope, interceptMean);
      }
      final RealVar __interceptSD = interceptSD;
      RealVar slopeMean;
      if (this.slopeMean != null && this.slopeMean.isPresent()) {
        slopeMean = this.slopeMean.get();
      } else {
        slopeMean = $generated__29(data, rocketType, launch, success, intercept, slope, interceptMean, interceptSD);
      }
      final RealVar __slopeMean = slopeMean;
      RealVar slopeSD;
      if (this.slopeSD != null && this.slopeSD.isPresent()) {
        slopeSD = this.slopeSD.get();
      } else {
        slopeSD = $generated__30(data, rocketType, launch, success, intercept, slope, interceptMean, interceptSD, slopeMean);
      }
      final RealVar __slopeSD = slopeSD;
      Map<String, RealVar> prediction;
      if (this.prediction != null && this.prediction.isPresent()) {
        prediction = this.prediction.get();
      } else {
        prediction = $generated__31(data, rocketType, launch, success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD);
      }
      final Map<String, RealVar> __prediction = prediction;
      // Build the instance after boxing params
      return new GLM(
        __success, 
        __intercept, 
        __slope, 
        __interceptMean, 
        __interceptSD, 
        __slopeMean, 
        __slopeSD, 
        __prediction, 
        new ConstantSupplier(__data), 
        new ConstantSupplier(__rocketType), 
        new ConstantSupplier(__launch)
      );
    }
  }
  
  @DesignatedConstructor
  public static GLM.Builder builderFromCommandLine() {
    Builder result = new Builder();
    result.fromCommandLine = true;
    return result;
  }
  
  @Param
  private final Supplier<GlobalDataSource> $generated__data;
  
  public GlobalDataSource getData() {
    return $generated__data.get();
  }
  
  @Param
  private final Supplier<Plate<String>> $generated__rocketType;
  
  public Plate<String> getRocketType() {
    return $generated__rocketType.get();
  }
  
  @Param
  private final Supplier<Plate<Integer>> $generated__launch;
  
  public Plate<Integer> getLaunch() {
    return $generated__launch.get();
  }
  
  private final Plated<IntVar> success;
  
  public Plated<IntVar> getSuccess() {
    return success;
  }
  
  private final Plated<RealVar> intercept;
  
  public Plated<RealVar> getIntercept() {
    return intercept;
  }
  
  private final Plated<RealVar> slope;
  
  public Plated<RealVar> getSlope() {
    return slope;
  }
  
  private final RealVar interceptMean;
  
  public RealVar getInterceptMean() {
    return interceptMean;
  }
  
  private final RealVar interceptSD;
  
  public RealVar getInterceptSD() {
    return interceptSD;
  }
  
  private final RealVar slopeMean;
  
  public RealVar getSlopeMean() {
    return slopeMean;
  }
  
  private final RealVar slopeSD;
  
  public RealVar getSlopeSD() {
    return slopeSD;
  }
  
  private final Map<String, RealVar> prediction;
  
  public Map<String, RealVar> getPrediction() {
    return prediction;
  }
  
  /**
   * Utility main method for posterior inference on this model
   */
  public static void main(final String[] arguments) {
    StaticJavaUtils.callRunner(Builder.class, arguments);
  }
  
  /**
   * Auxiliary method generated to translate:
   * interceptMean
   */
  private static RealVar $generated__0(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    return interceptMean;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__1() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__1_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__1();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__1_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0
   */
  private static RealVar $generated__2() {
    return new blang.core.RealConstant(0);
  }
  
  public static class $generated__2_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__2();
    }
    
    public String toString() {
      return "0";
    }
    
    public $generated__2_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__3() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__3_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__3();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__3_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * slopeMean
   */
  private static RealVar $generated__4(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    return slopeMean;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__5() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__5_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__5();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__5_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 0
   */
  private static RealVar $generated__6() {
    return new blang.core.RealConstant(0);
  }
  
  public static class $generated__6_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__6();
    }
    
    public String toString() {
      return "0";
    }
    
    public $generated__6_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__7() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__7_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__7();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__7_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * interceptSD
   */
  private static RealVar $generated__8(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    return interceptSD;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__9() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__9_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__9();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__9_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__10() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__10_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__10();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__10_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * slopeSD
   */
  private static RealVar $generated__11(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    return slopeSD;
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__12() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__12_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__12();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__12_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * 1
   */
  private static RealVar $generated__13() {
    return new blang.core.RealConstant(1);
  }
  
  public static class $generated__13_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__13();
    }
    
    public String toString() {
      return "1";
    }
    
    public $generated__13_class() {
      
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * rocketType.indices
   */
  private static Iterable<Index<String>> $generated__14(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    Collection<Index<String>> _indices = rocketType.indices();
    return _indices;
  }
  
  /**
   * Auxiliary method generated to translate:
   * intercept.get(rocket)
   */
  private static RealVar $generated__15(final Index<String> rocket, final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    RealVar _get = intercept.get(rocket);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * interceptMean
   */
  private static RealVar $generated__16(final RealVar interceptMean, final RealVar interceptSD) {
    return interceptMean;
  }
  
  public static class $generated__16_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__16(interceptMean, interceptSD);
    }
    
    public String toString() {
      return "interceptMean";
    }
    
    private final RealVar interceptMean;
    
    private final RealVar interceptSD;
    
    public $generated__16_class(final RealVar interceptMean, final RealVar interceptSD) {
      this.interceptMean = interceptMean;
      this.interceptSD = interceptSD;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * pow(interceptSD, 2)
   */
  private static RealVar $generated__17(final RealVar interceptMean, final RealVar interceptSD) {
    double _pow = Math.pow((interceptSD).doubleValue(), 2);
    return new blang.core.RealConstant(_pow);
  }
  
  public static class $generated__17_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__17(interceptMean, interceptSD);
    }
    
    public String toString() {
      return "pow(interceptSD, 2)";
    }
    
    private final RealVar interceptMean;
    
    private final RealVar interceptSD;
    
    public $generated__17_class(final RealVar interceptMean, final RealVar interceptSD) {
      this.interceptMean = interceptMean;
      this.interceptSD = interceptSD;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * slope.get(rocket)
   */
  private static RealVar $generated__18(final Index<String> rocket, final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    RealVar _get = slope.get(rocket);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * slopeMean
   */
  private static RealVar $generated__19(final RealVar slopeMean, final RealVar slopeSD) {
    return slopeMean;
  }
  
  public static class $generated__19_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__19(slopeMean, slopeSD);
    }
    
    public String toString() {
      return "slopeMean";
    }
    
    private final RealVar slopeMean;
    
    private final RealVar slopeSD;
    
    public $generated__19_class(final RealVar slopeMean, final RealVar slopeSD) {
      this.slopeMean = slopeMean;
      this.slopeSD = slopeSD;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * pow(slopeSD, 2)
   */
  private static RealVar $generated__20(final RealVar slopeMean, final RealVar slopeSD) {
    double _pow = Math.pow((slopeSD).doubleValue(), 2);
    return new blang.core.RealConstant(_pow);
  }
  
  public static class $generated__20_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__20(slopeMean, slopeSD);
    }
    
    public String toString() {
      return "pow(slopeSD, 2)";
    }
    
    private final RealVar slopeMean;
    
    private final RealVar slopeSD;
    
    public $generated__20_class(final RealVar slopeMean, final RealVar slopeSD) {
      this.slopeMean = slopeMean;
      this.slopeSD = slopeSD;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * launch.indices(rocket)
   */
  private static Iterable<Index<Integer>> $generated__21(final Index<String> rocket, final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    Collection<Index<Integer>> _indices = launch.indices(rocket);
    return _indices;
  }
  
  /**
   * Auxiliary method generated to translate:
   * launchIndex.key
   */
  private static int $generated__22(final Index<Integer> launchIndex, final Index<String> rocket, final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    return (launchIndex.key).intValue();
  }
  
  /**
   * Auxiliary method generated to translate:
   * intercept.get(rocket)
   */
  private static RealVar $generated__23(final Index<Integer> launchIndex, final Index<String> rocket, final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    RealVar _get = intercept.get(rocket);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * slope.get(rocket)
   */
  private static RealVar $generated__24(final Index<Integer> launchIndex, final Index<String> rocket, final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    RealVar _get = slope.get(rocket);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * success.get(rocket, launchIndex)
   */
  private static IntVar $generated__25(final Index<Integer> launchIndex, final Index<String> rocket, final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD, final Map<String, RealVar> prediction) {
    IntVar _get = success.get(rocket, launchIndex);
    return _get;
  }
  
  /**
   * Auxiliary method generated to translate:
   * logistic(beta0 + beta1 * x)
   */
  private static RealVar $generated__26(final int x, final RealVar beta0, final RealVar beta1) {
    double _plus = BlangExtensions.operator_plus(beta0, Double.valueOf(((beta1).doubleValue() * x)));
    double _logistic = SpecialFunctions.logistic(_plus);
    return new blang.core.RealConstant(_logistic);
  }
  
  public static class $generated__26_class implements Supplier<RealVar> {
    public RealVar get() {
      return $generated__26(x, beta0, beta1);
    }
    
    public String toString() {
      return "logistic(beta0 + beta1 * x)";
    }
    
    private final int x;
    
    private final RealVar beta0;
    
    private final RealVar beta1;
    
    public $generated__26_class(final int x, final RealVar beta0, final RealVar beta1) {
      this.x = x;
      this.beta0 = beta0;
      this.beta1 = beta1;
    }
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentReal
   */
  private static RealVar $generated__27(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope) {
    RealScalar _latentReal = StaticUtils.latentReal();
    return _latentReal;
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentReal
   */
  private static RealVar $generated__28(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean) {
    RealScalar _latentReal = StaticUtils.latentReal();
    return _latentReal;
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentReal
   */
  private static RealVar $generated__29(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD) {
    RealScalar _latentReal = StaticUtils.latentReal();
    return _latentReal;
  }
  
  /**
   * Auxiliary method generated to translate:
   * latentReal
   */
  private static RealVar $generated__30(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean) {
    RealScalar _latentReal = StaticUtils.latentReal();
    return _latentReal;
  }
  
  /**
   * Auxiliary method generated to translate:
   * Utils.prediction(rocketType, launch, intercept, slope)
   */
  private static Map<String, RealVar> $generated__31(final GlobalDataSource data, final Plate<String> rocketType, final Plate<Integer> launch, final Plated<IntVar> success, final Plated<RealVar> intercept, final Plated<RealVar> slope, final RealVar interceptMean, final RealVar interceptSD, final RealVar slopeMean, final RealVar slopeSD) {
    Map<String, RealVar> _prediction = Utils.prediction(rocketType, launch, intercept, slope);
    return _prediction;
  }
  
  /**
   * Note: the generated code has the following properties used at runtime:
   *   - all arguments are annotated with a BlangVariable annotation
   *   - params additionally have a Param annotation
   *   - the order of the arguments is as follows:
   *     - first, all the random variables in the order they occur in the blang file
   *     - second, all the params in the order they occur in the blang file
   * 
   */
  public GLM(@DeboxedName("success") final Plated<IntVar> success, @DeboxedName("intercept") final Plated<RealVar> intercept, @DeboxedName("slope") final Plated<RealVar> slope, @DeboxedName("interceptMean") final RealVar interceptMean, @DeboxedName("interceptSD") final RealVar interceptSD, @DeboxedName("slopeMean") final RealVar slopeMean, @DeboxedName("slopeSD") final RealVar slopeSD, @DeboxedName("prediction") final Map<String, RealVar> prediction, @Param @DeboxedName("data") final Supplier<GlobalDataSource> $generated__data, @Param @DeboxedName("rocketType") final Supplier<Plate<String>> $generated__rocketType, @Param @DeboxedName("launch") final Supplier<Plate<Integer>> $generated__launch) {
    this.$generated__data = $generated__data;
    this.$generated__rocketType = $generated__rocketType;
    this.$generated__launch = $generated__launch;
    this.success = success;
    this.intercept = intercept;
    this.slope = slope;
    this.interceptMean = interceptMean;
    this.interceptSD = interceptSD;
    this.slopeMean = slopeMean;
    this.slopeSD = slopeSD;
    this.prediction = prediction;
  }
  
  /**
   * A component can be either a distribution, support constraint, or another model  
   * which recursively defines additional components.
   */
  public Collection<ModelComponent> components() {
    ArrayList<ModelComponent> components = new ArrayList();
    
    { // Code generated by: interceptMean ~ StudentT(1, 0, 1)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.StudentT(
          $generated__0($generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction), 
          new $generated__1_class(), 
          new $generated__2_class(), 
          new $generated__3_class()
        )
        );
    }
    { // Code generated by: slopeMean ~ StudentT(1, 0, 1)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.StudentT(
          $generated__4($generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction), 
          new $generated__5_class(), 
          new $generated__6_class(), 
          new $generated__7_class()
        )
        );
    }
    { // Code generated by: interceptSD ~ HalfStudentT(1, 1)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.HalfStudentT(
          $generated__8($generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction), 
          new $generated__9_class(), 
          new $generated__10_class()
        )
        );
    }
    { // Code generated by: slopeSD ~ HalfStudentT(1, 1)
      // Construction and addition of the factor/model:
      components.add(
        new blang.distributions.HalfStudentT(
          $generated__11($generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction), 
          new $generated__12_class(), 
          new $generated__13_class()
        )
        );
    }
    for (Index<String> rocket : $generated__14($generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction)) {
      { // Code generated by: intercept.get(rocket) | interceptMean, interceptSD ~ Normal(interceptMean, pow(interceptSD, 2))
        // Construction and addition of the factor/model:
        components.add(
          new blang.distributions.Normal(
            $generated__15(rocket, $generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction), 
            new $generated__16_class(interceptMean, interceptSD), 
            new $generated__17_class(interceptMean, interceptSD)
          )
          );
      }
      { // Code generated by: slope.get(rocket) | slopeMean, slopeSD ~ Normal(slopeMean, pow(slopeSD, 2))
        // Construction and addition of the factor/model:
        components.add(
          new blang.distributions.Normal(
            $generated__18(rocket, $generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction), 
            new $generated__19_class(slopeMean, slopeSD), 
            new $generated__20_class(slopeMean, slopeSD)
          )
          );
      }
      for (Index<Integer> launchIndex : $generated__21(rocket, $generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction)) {
        { // Code generated by: success.get(rocket, launchIndex) | int x = launchIndex.key, RealVar beta0 = intercept.get(rocket), RealVar beta1 = slope.get(rocket) ~ Bernoulli(logistic(beta0 + beta1 * x))
          // Required initialization:
          int x = $generated__22(launchIndex, rocket, $generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction);
          RealVar beta0 = $generated__23(launchIndex, rocket, $generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction);
          RealVar beta1 = $generated__24(launchIndex, rocket, $generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction);
          // Construction and addition of the factor/model:
          components.add(
            new blang.distributions.Bernoulli(
              $generated__25(launchIndex, rocket, $generated__data.get(), $generated__rocketType.get(), $generated__launch.get(), success, intercept, slope, interceptMean, interceptSD, slopeMean, slopeSD, prediction), 
              new $generated__26_class(x, beta0, beta1)
            )
            );
        }
      }
    }
    
    return components;
  }
}
