require("ggplot2")
require("dplyr")

data <- read.csv("/Users/bouchard/w/blogobayes/www/ex/hm/results/all/Basic-645138973/samples/failureProbability.csv")

# ggplot has bad default sizes for large facetting
verticalSize <- 2.0 *  1 
horizontalSize <- 4.0

n_samples <- max(data$sample)
cut_off <- n_samples * 0.5
data <- subset(data, sample > cut_off)
hdi_upper <- function(samples) {
	n = length(samples)
	m = as.integer(0.9 * n)
	sorted_samples <- sort(samples)
  shortest_length <- Inf
  shortest_interval <- c()
  for (i in 1:(n-m)){
    lower <- sorted_samples[i]
    upper <- sorted_samples[i+m]
    interval_length <- upper - lower
    if (interval_length < shortest_length) {
      shortest_length <- interval_length
      shortest_interval <- c(lower, upper)
    }
  }
return (shortest_interval[2])
}
hdi_lower <- function(samples) {
	n = length(samples)
	m = as.integer(0.9 * n)
	sorted_samples <- sort(samples)
  shortest_length <- Inf
  shortest_interval <- c()
  for (i in 1:(n-m)){
    lower <- sorted_samples[i]
    upper <- sorted_samples[i+m]
    interval_length <- upper - lower
    if (interval_length < shortest_length) {
      shortest_length <- interval_length
      shortest_interval <- c(lower, upper)
    }
  }
return (shortest_interval[1])
}
hdi_df <- data %>% group_by() %>%
 summarise(HDI.lower=hdi_lower(value),
           HDI.upper=hdi_upper(value))

p <- ggplot(data, aes(x = value)) +
  geom_density() + 
  theme_bw() + 
  geom_segment(inherit.aes = FALSE, data = hdi_df, aes(x=HDI.lower, xend=HDI.upper, y=0, yend=0), col="red") + 
  geom_point(inherit.aes = FALSE, data = hdi_df, aes(x=HDI.lower, y=0), col="red") + 
  geom_point(inherit.aes = FALSE, data = hdi_df, aes(x=HDI.upper, y=0), col="red") + 
  geom_text(inherit.aes = FALSE, data = hdi_df, aes(y= 0, label="0.9-HDI", x=(HDI.lower + HDI.upper)/2), vjust=-0.5) +
  xlab("failureProbability") +
  ylab("density") +
  ggtitle("Density plot for: failureProbability")
ggsave("/Users/bouchard/w/blogobayes/www/ex/hm/results/all/Basic-645138973/posteriorPlots/failureProbability.pdf", limitsize = F, height = verticalSize, width = horizontalSize)
